import sys
import os
import copy

class Storages:
    def __init__(self, table, bowls, cups, max_cups):
        self.table = table
        self.bowls = bowls
        self.cups = cups
        self.max_cups = max_cups

class Contexts:
    def __init__(self, dough, chocolate, lemons, bowl_filling, mixing, cup_filling, baking):
        self.dough = dough
        self.chocolate = chocolate
        self.lemons = lemons
        self.bowl_filling = bowl_filling
        self.mixing = mixing
        self.cup_filling = cup_filling
        self.baking = baking
        
    def __getitem__(self, index):
        return [self.dough, self.chocolate, self.lemons, self.bowl_filling, self.mixing, self.cup_filling, self.baking][index]

class Mouse:
    def __init__(
            self,
            id,
            dough_fetching_speed,
            chocolate_fetching_speed,
            lemon_fetching_speed,
            bowl_filling_speed,
            mixing_speed,
            cup_filling_speed,
            baking_speed):
        self.id = id
        self.context = None
        self.dough_fetching_speed = dough_fetching_speed
        self.chocolate_fetching_speed = chocolate_fetching_speed
        self.lemon_fetching_speed = lemon_fetching_speed
        self.bowl_filling_speed = bowl_filling_speed
        self.mixing_speed = mixing_speed
        self.cup_filling_speed = cup_filling_speed
        self.baking_speed = baking_speed

class Context:
    def __init__(self, id, switch_cost):
        self.id = id
        self.switch_cost = switch_cost

class Dough(Context):
    def __init__(self, switch_cost, max_amount):
        Context.__init__(self, 0, switch_cost)
        self.max_amount = max_amount

class Chocolate(Context):
    def __init__(self, switch_cost, max_amount):
        Context.__init__(self, 1, switch_cost)
        self.max_amount = max_amount

class Lemons(Context):
    def __init__(self, switch_cost, max_amount):
        Context.__init__(self, 2, switch_cost)
        self.max_amount = max_amount

class BowlFilling(Context):
    def __init__(self, switch_cost):
        Context.__init__(self, 3, switch_cost)

class Mixing(Context):
    def __init__(self, switch_cost):
        Context.__init__(self, 4, switch_cost)

class CupFilling(Context):
    def __init__(self, switch_cost):
        Context.__init__(self, 5, switch_cost)

class Baking(Context):
    def __init__(self, switch_cost, baking_time):
        Context.__init__(self, 6, switch_cost)
        self.baking_time = baking_time

class Bowl:
    def __init__(self, capacity, id):
        self.capacity = capacity
        self.amount = 0
        self.resource = 0
        self.mixed = False
        self.in_use = False
        self.id = id

    def update(self, amount, resource, mixed, in_use):
        self.amount = int(amount)
        self.resource = int(resource)
        self.mixed = bool(int(mixed))
        self.in_use = bool(int(in_use))
        
class Order:
    def __init__(self, muffin_type, tick, amount):
        self.muffin_type = muffin_type
        self.tick = tick
        self.amount = amount

def update_containers():
    # update containers
    storages.table[:] = [int(x) for x in input().split()]
    for bowl in storages.bowls:
        bowl.update(*input().split())
    storages.cups[1], storages.cups[2] = [int(x) for x in input().split()]

def process_events():

    recv = input().split()
    if recv[0] == 'T':
        global current_tick, current_turn
        current_tick = int(recv[1])
        current_turn = int(recv[2])
        
        if current_turn == my_id:
            update_containers()
            return
        process_events()
    elif recv[0] == 'A':
        action = int(recv[1])
        mouse = mice[current_turn]
        old_storages = copy.deepcopy(storages)
        update_containers()
        
        if action == 0:
            # switch context
            context_id = int(recv[2])
            busy_until = int(recv[3])
            context_callins = [bot.mouse_going_to_dough_station, bot.mouse_going_to_chocolate_station, bot.mouse_going_to_lemon_station, bot.mouse_going_to_bowl_filling_station, bot.mouse_going_to_mixing_station, bot.mouse_going_to_cup_filling_station, bot.mouse_going_to_baking_station]
            context_callins[context_id](mouse, busy_until)
            mouse.context = contexts[context_id]
            
        if action == 1:
            # execute context
            
            finish_tick = int(recv[-1])
            context_id = mouse.context.id
            
            if context_id == 0:
                bot.mouse_getting_dough(mouse, finish_tick)
            if context_id == 1:
                bot.mouse_getting_chocolate(mouse, finish_tick)
            if context_id == 2:
                bot.mouse_getting_lemons(mouse, finish_tick)
            if context_id == 3:
                bowl = storages.bowls[int(recv[2])]
                resource_id = int(recv[3])
                bot.mouse_filling_bowls(mouse, bowl, resource_id, old_storages, finish_tick)
            if context_id == 4:
                bowl = storages.bowls[int(recv[2])]
                bot.mouse_mixing_bowl(mouse, bowl, finish_tick)
            if context_id == 5:
                bowl = storages.bowls[int(recv[2])]
                bot.mouse_filling_cups(mouse, bowl, old_storages, finish_tick)
            if context_id == 6:
                bot.mouse_baking_muffins(mouse, old_storages, finish_tick)
            
        if action == 2:
            # idle
            finish_tick = int(recv[-1])
            bot.mouse_idling(mouse, finish_tick)
        process_events()
    elif recv[0] == 'E':
        sys.exit(0)    
    else:
        assert false, f"Unknown input {recv}"

def print_to_grader(*args, **kwargs):
    print(*args, file=sys.__stdout__, **kwargs, flush=True)
        
def idle(until=-1):
    print_to_grader(2, until)
    process_events()
    
def get_dough():
    assert mice[my_id].context == contexts.dough, "Can't get dough when not at dough station"
    print_to_grader(1)
    process_events()

def get_chocolate():
    assert mice[my_id].context == contexts.chocolate, "Can't get chocolate when not at chocolate station"
    print_to_grader(1)
    process_events()

def get_lemons():
    assert mice[my_id].context == contexts.lemons, "Can't get lemons when not at lemon station"
    print_to_grader(1)
    process_events()

def fill_bowl(bowl, resource_id):
    assert mice[my_id].context == contexts.bowl_filling, "Can't fill bowls when not at bowl filling station"
    assert 0 <= bowl.id < len(storages.bowls), f"Invalid bowl id {bowl.id}, it must be in [0, {len(storages.bowls) - 1}]"
    print_to_grader(1, bowl.id, resource_id)
    process_events()
    
def mix_bowl(bowl):
    assert mice[my_id].context == contexts.mixing, "Can't mix bowls when not at mixing station"
    assert 0 <= bowl.id < len(storages.bowls), f"Invalid bowl id {bowl.id}, it must be in [0, {len(storages.bowls) - 1}]"
    print_to_grader(1, bowl.id)
    process_events()

def fill_cups(bowl):
    assert mice[my_id].context == contexts.cup_filling, "Can't fill cups when not at cup filling station"
    assert 0 <= bowl.id < len(storages.bowls), f"Invalid bowl id {bowl.id}, it must be in [0, {len(storages.bowls) - 1}]"
    print_to_grader(1, bowl.id)
    process_events()
    
def bake_muffins():
    assert mice[my_id].context == contexts.baking, "Can't bake muffins when not at baking station"
    print_to_grader(1)
    process_events()
    
def go_to(context):
    print_to_grader(0, context.id)
    process_events()

def get_current_tick():
    return current_tick
    
def run(Bot):
    global storages, contexts, bot, mice, my_id, orders, current_tick

    sys.stdout = sys.stderr
    current_tick = 0
    num_bowls, capacity = map(int,input().split())
    max_cups = int(input())
    bowls = [Bowl(capacity, id) for id in range(num_bowls)]
    cups = {1: 0, 2: 0}
    table = [0, 0, 0]
    dough = Dough(*map(int, input().split()))
    chocolate = Chocolate(*map(int, input().split()))
    lemons = Lemons(*map(int, input().split()))
    bowl_filling = BowlFilling(*map(int, input().split()))
    mixing = Mixing(*map(int, input().split()))
    cup_filling = CupFilling(*map(int, input().split()))
    baking = Baking(*map(int, input().split()))

    num_mice = int(input())
    mice = []
    for i in range(num_mice):
        mice.append(Mouse(*([i] + list(map(int, input().split())))))

    my_id = int(input())
    num_orders = int(input())
    orders = []
    for i in range(num_orders):
        orders.append(Order(*map(int, input().split())))
    storages = Storages(table, bowls, cups, max_cups)
    contexts = Contexts(dough, chocolate, lemons, bowl_filling, mixing, cup_filling, baking)
    bot = Bot(storages, contexts, mice, my_id, orders)
    process_events()
    bot.main()
